---
title: 接收请求
---
::: tip
请参考[附录5](./附录5%20初始化中间件){target="_blank"}之后，再来看这篇。
:::

# 接收请求

由前一节可知，事件的注册在`Hyerf\Server\Server`类的`registerSwooleEvents`方法中完成。

![](https://oss.xiaokeaii.top/2024/event.png)

注册的回调事件的执行顺序如下：
| 所属类 | 事件 | 描述 | 触发时机 | 触发顺序 |
| --- | --- | --- | --- | --- |
| StartCallback | onStart | 当 swoole_server 实例启动后触发 | 仅当 master 进程启动成功后触发 | 1 |
| ManagerStartCallback | onManagerStart | 当 swoole_server 实例启动后触发 | 仅当 manager 进程启动成功后触发 | 2 |
| WorkerStartCallback | onWorkerStart | 当 worker 进程启动后触发 | 仅当 worker 进程启动成功后触发 | 3 |
| WorkerStopCallback | onWorkerStop | 当 worker 进程关闭后触发 | 仅当 worker 进程关闭后触发 | 4 |
| WorkerExitCallback | onWorkerExit | 当 worker 进程退出后触发 | 仅当 worker 进程退出后触发 | 5 |
| Server | onRequest | 当接收到请求时触发 | 仅当 worker 进程接收到请求后触发 | 6 |
| PipeMessageCallback | onPipeMessage | 当接收到消息时触发 | 仅当 worker 进程接收到消息后触发 | 7 |
::: tip
 要理解上面事件，请参考`Swoole`的运行流程。
:::

### Swoole流程

首先了解`Swoole`的运行流程，(图源网络)
![](https://oss.xiaokeaii.top/2024/swoole流程.jpg)
具体内容参考[swoole官网](https://wiki.swoole.com/)

::: tip
当请求过来时，会触发`Request`事件，基于之前准备阶段注册的`swoole`事件可知，会调用`Hyperf\HttpServer\Server`类的`onRequest`函数。
:::

### OnRequest方法

```php
public function onRequest($request, $response): void
{
    try {
        // 等待worker进程启动
        CoordinatorManager::until(Constants::WORKER_START)->yield();
        // 请求和响应对象转换为符合 PSR-7标准的对象($request, $response为swoole返回的对象)
        [$psr7Request, $psr7Response] = $this->initRequestAndResponse($request, $response);
        // 路由匹配
        $psr7Request = $this->coreMiddleware->dispatch($psr7Request);

        $this->option?->isEnableRequestLifecycle() && $this->event?->dispatch(new RequestReceived(
            request: $psr7Request,
            response: $psr7Response,
            server: $this->serverName
        ));

        /** @var Dispatched $dispatched */
        // 获取匹配的路由信息
        $dispatched = $psr7Request->getAttribute(Dispatched::class);
        // 获取配置文件中定义的路由
        $middlewares = $this->middlewares;
        $registeredMiddlewares = [];
        // 路由存在的情况
        if ($dispatched->isFound()) {
            // 获取路由注册的中间件
            $registeredMiddlewares = MiddlewareManager::get($this->serverName, $dispatched->handler->route, $psr7Request->getMethod());
            // 跟配置文件合并
            $middlewares = array_merge($middlewares, $registeredMiddlewares);
        }
        // 如果mustSortMiddlewares参数为true或者registeredMiddlewares中间件存在，则进行排序操作
        if ($this->option?->isMustSortMiddlewares() || $registeredMiddlewares) {
            $middlewares = MiddlewareManager::sortMiddlewares($middlewares);
        }

        // 获取响应信息
        $psr7Response = $this->dispatcher->dispatch($psr7Request, $middlewares, $this->coreMiddleware);
    } catch (Throwable $throwable) {
        // Delegate the exception to exception handler.
        $psr7Response = $this->container->get(SafeCaller::class)->call(function () use ($throwable) {
            return $this->exceptionHandlerDispatcher->dispatch($throwable, $this->exceptionHandlers);
        }, static function () {
            return (new Psr7Response())->withStatus(400);
        });
    } finally {
        // isEnableRequestLifecycle默认不开启
        if (isset($psr7Request) && $this->option?->isEnableRequestLifecycle()) {
            defer(fn () => $this->event?->dispatch(new RequestTerminated(
                request: $psr7Request,
                response: $psr7Response ?? null,
                exception: $throwable ?? null,
                server: $this->serverName
            )));

            $this->event?->dispatch(new RequestHandled(
                request: $psr7Request,
                response: $psr7Response ?? null,
                exception: $throwable ?? null,
                server: $this->serverName
            ));
        }

        // Send the Response to client.
        if (! isset($psr7Response) || ! $psr7Response instanceof ResponseInterface) {
            return;
        }
        // 发送响应到客户端
        if (isset($psr7Request) && $psr7Request->getMethod() === 'HEAD') {
            $this->responseEmitter->emit($psr7Response, $response, false);
        } else {
            $this->responseEmitter->emit($psr7Response, $response);
        }
    }
}
```

`CoordinatorManager::until`用于创建一个协程等待器，指示协程等待某个事件的完成,
这里是等待`worker`进程启动完成。

```php
CoordinatorManager::until(Constants::WORKER_START)->yield();
```

确保在处理请求之前，所有的`Worker`进程都已经启动完成，以保证后续的操作能够顺利执行。

:::code-group

```php [转换为标准对象]
[$psr7Request, $psr7Response] = $this->initRequestAndResponse($request, $response);
```

```php [转换实现]
protected function initRequestAndResponse($request, $response): array
{
    // 设置到`ResponseContext`中，
    ResponseContext::set($psr7Response = new Psr7Response());

    $psr7Response->setConnection(new WritableConnection($response));

    if ($request instanceof ServerRequestInterface) {
        $psr7Request = $request;
    } else {
        $psr7Request = Psr7Request::loadFromSwooleRequest($request);
    }
    // 设置请求到RequestContext中
    RequestContext::set($psr7Request);

    return [$psr7Request, $psr7Response];
}
```

:::
创建一个新的 `Psr7Response` 实例，并将其设置到 `ResponseContext` 上下文中。`ResponseContext`和`RequestContext` 是静态类，用于在全局范围内存储响应对象。



#### 路由寻址

请求和响应对象转换完成之后，开始匹配路由，
::: code-group
```php [寻址]
$psr7Request = $this->coreMiddleware->dispatch($psr7Request);
```
```php [路由查找]
public function dispatch(ServerRequestInterface $request): ServerRequestInterface
{
    // 进行查找路由操作
    $routes = $this->dispatcher->dispatch($request->getMethod(), $request->getUri()->getPath());

    $dispatched = new Dispatched($routes, $this->serverName);

    return RequestContext::set($request)->setAttribute(Dispatched::class, $dispatched);
}

```

```php [匹配实现]
public function dispatch($httpMethod, $uri)
{
    if (isset($this->staticRouteMap[$httpMethod][$uri])) {
        $handler = $this->staticRouteMap[$httpMethod][$uri];
        return [self::FOUND, $handler, []];
    }

    $varRouteData = $this->variableRouteData;
    if (isset($varRouteData[$httpMethod])) {
        $result = $this->dispatchVariableRoute($varRouteData[$httpMethod], $uri);
        if ($result[0] === self::FOUND) {
            return $result;
        }
    }

    // For HEAD requests, attempt fallback to GET
    if ($httpMethod === 'HEAD') {
        if (isset($this->staticRouteMap['GET'][$uri])) {
            $handler = $this->staticRouteMap['GET'][$uri];
            return [self::FOUND, $handler, []];
        }
        if (isset($varRouteData['GET'])) {
            $result = $this->dispatchVariableRoute($varRouteData['GET'], $uri);
            if ($result[0] === self::FOUND) {
                return $result;
            }
        }
    }

    // If nothing else matches, try fallback routes
    if (isset($this->staticRouteMap['*'][$uri])) {
        $handler = $this->staticRouteMap['*'][$uri];
        return [self::FOUND, $handler, []];
    }
    if (isset($varRouteData['*'])) {
        $result = $this->dispatchVariableRoute($varRouteData['*'], $uri);
        if ($result[0] === self::FOUND) {
            return $result;
        }
    }

    // Find allowed methods for this URI by matching against all other HTTP methods as well
    $allowedMethods = [];

    foreach ($this->staticRouteMap as $method => $uriMap) {
        if ($method !== $httpMethod && isset($uriMap[$uri])) {
            $allowedMethods[] = $method;
        }
    }

    foreach ($varRouteData as $method => $routeData) {
        if ($method === $httpMethod) {
            continue;
        }

        $result = $this->dispatchVariableRoute($routeData, $uri);
        if ($result[0] === self::FOUND) {
            $allowedMethods[] = $method;
        }
    }

    // If there are no allowed methods the route simply does not exist
    if ($allowedMethods) {
        return [self::METHOD_NOT_ALLOWED, $allowedMethods];
    }

    return [self::NOT_FOUND];
}
```

:::

当匹配到时，返回参数如下所示，数组第一个参数为标识是否匹配到路由的状态码。

```php
Array
(
    [0] => 1
    [1] => Hyperf\HttpServer\Router\Handler Object
        (
            [callback] => Array
                (
                    [0] => App\Controller\IndexController
                    [1] => index
                )

            [route] => /index/index
            [options] => Array
                (
                    [middleware] => Array
                        (
                        )

                )

        )

    [2] => Array
        (
        )

)
```

```php
$dispatched = new Dispatched($routes, $this->serverName);
```

使用获取到的路由信息数组和当前`Server`实例化一个`Dispatched`类来保存匹配信息。

```php
namespace Hyperf\HttpServer\Router;

use FastRoute\Dispatcher;

class Dispatched
{
    public int $status;

    public ?Handler $handler = null;

    public array $params = [];

    /**
     * 根据匹配信息，设置实例参数
     *
     * @param array $array with one of the following formats:
     *
     *     [Dispatcher::NOT_FOUND]
     *     [Dispatcher::METHOD_NOT_ALLOWED, ['GET', 'OTHER_ALLOWED_METHODS']]
     *     [Dispatcher::FOUND, $handler, ['varName' => 'value', ...]]
     */
    public function __construct(array $array, public ?string $serverName = null)
    {
        $this->status = $array[0];
        switch ($this->status) {
            case Dispatcher::METHOD_NOT_ALLOWED:
                $this->params = $array[1];
                break;
            case Dispatcher::FOUND:
                $this->handler = $array[1];
                $this->params = $array[2];
                break;
        }
    }

    public function isFound(): bool
    {
        return $this->status === Dispatcher::FOUND;
    }

    public function isNotFound(): bool
    {
        return $this->status === Dispatcher::NOT_FOUND;
    }
}
```

最后设置请求上下文，并返回修改后的请求对象。

```php
return RequestContext::set($request)->setAttribute(Dispatched::class, $dispatched);
```

```php
$this->option?->isEnableRequestLifecycle() && $this->event?->dispatch(new RequestReceived(
    request: $psr7Request,
    response: $psr7Response,
    server: $this->serverName
));
```

如果`option`参数设置了`enable_request_lifecycle`为`true`，则触发`RequestReceived`事件。

```php
$dispatched = $psr7Request->getAttribute(Dispatched::class);
```

从请求中取出刚才设置的`Dispatched`实例。

#### 处理中间件

首先获取配置文件中的路由，
如果路由匹配成功，则获取路由上的中间件配置，然后将中间件进行合并。
```php
$middlewares = $this->middlewares;
$registeredMiddlewares = [];
if ($dispatched->isFound()) {
    $registeredMiddlewares = MiddlewareManager::get($this->serverName, $dispatched->handler->route, $psr7Request->getMethod());
    $middlewares = array_merge($middlewares, $registeredMiddlewares);
}
// 如果mustSortMiddlewares参数为true或者registeredMiddlewares中间件存在，则进行排序操作
if ($this->option?->isMustSortMiddlewares() || $registeredMiddlewares) {
    $middlewares = MiddlewareManager::sortMiddlewares($middlewares);
}
```
