---
title: 附录5 初始化中间件
---
# 初始化中间件

在注册服务,注册服务的事件时，有一个初始化中间件的操作。

```php
protected function registerSwooleEvents(SwoolePort|SwooleServer $server, array $events, string $serverName): void
{
    foreach ($events as $event => $callback) {
        if (! Event::isSwooleEvent($event)) {
            continue;
        }
        if (is_array($callback)) {
            // 获取事件对应class和回调方法
            [$className, $method] = $callback;
            if (array_key_exists($className . $method, $this->onRequestCallbacks)) {
                $this->logger->warning(sprintf('%s will be replaced by %s. Each server should have its own onRequest callback. Please check your configs.', $this->onRequestCallbacks[$className . $method], $serverName));
            }
            // 记录回调
            $this->onRequestCallbacks[$className . $method] = $serverName;
            // 解析回调类
            $class = $this->container->get($className);
            // 设置服务名
            if (method_exists($class, 'setServerName')) {
                // Override the server name.
                $class->setServerName($serverName);
            }
            // 如果类实现MiddlewareInitializerInterface接口，则调用初始化中间件方法 // [!code focus:4]
            if ($class instanceof MiddlewareInitializerInterface) {
                $class->initCoreMiddleware($serverName);  
            }
            $callback = [$class, $method];
        }
        // 注册事件回调函数
        $server->on($event, $callback);
    }
}
```

其中，默认注册的回调事件有以下几种，我们看`request`回调，其中回调的类是`Hyperf\HttpServer\Server`类,
::: details 回调事件

```php
Array
(
    [start] => Array
        (
            [0] => Hyperf\Framework\Bootstrap\StartCallback
            [1] => onStart
        )

    [managerStart] => Array
        (
            [0] => Hyperf\Framework\Bootstrap\ManagerStartCallback
            [1] => onManagerStart
        )

    [workerStart] => Array
        (
            [0] => Hyperf\Framework\Bootstrap\WorkerStartCallback
            [1] => onWorkerStart
        )

    [workerStop] => Array
        (
            [0] => Hyperf\Framework\Bootstrap\WorkerStopCallback
            [1] => onWorkerStop
        )

    [workerExit] => Array
        (
            [0] => Hyperf\Framework\Bootstrap\WorkerExitCallback
            [1] => onWorkerExit
        )

    [pipeMessage] => Array
        (
            [0] => Hyperf\Framework\Bootstrap\PipeMessageCallback
            [1] => onPipeMessage
        )

    [request] => Array
        (
            [0] => Hyperf\HttpServer\Server
            [1] => onRequest
        )

)
```

:::
查看`Hyperf\HttpServer\Server`类，
::: details `Hyperf\HttpServer\Server`

```php
namespace Hyperf\HttpServer;

use FastRoute\Dispatcher;
use Hyperf\Context\RequestContext;
use Hyperf\Context\ResponseContext;
use Hyperf\Contract\ConfigInterface;
use Hyperf\Contract\MiddlewareInitializerInterface;
use Hyperf\Contract\OnRequestInterface;
use Hyperf\Coordinator\Constants;
use Hyperf\Coordinator\CoordinatorManager;
use Hyperf\Dispatcher\HttpDispatcher;
use Hyperf\Engine\Http\WritableConnection;
use Hyperf\ExceptionHandler\ExceptionHandlerDispatcher;
use Hyperf\HttpMessage\Server\Request as Psr7Request;
use Hyperf\HttpMessage\Server\Response as Psr7Response;
use Hyperf\HttpServer\Contract\CoreMiddlewareInterface;
use Hyperf\HttpServer\Event\RequestHandled;
use Hyperf\HttpServer\Event\RequestReceived;
use Hyperf\HttpServer\Event\RequestTerminated;
use Hyperf\HttpServer\Exception\Handler\HttpExceptionHandler;
use Hyperf\HttpServer\Router\Dispatched;
use Hyperf\HttpServer\Router\DispatcherFactory;
use Hyperf\Server\Option;
use Hyperf\Server\ServerFactory;
use Hyperf\Support\SafeCaller;
use Psr\Container\ContainerInterface;
use Psr\EventDispatcher\EventDispatcherInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Throwable;

use function Hyperf\Coroutine\defer;
use function Hyperf\Support\make;

//该类实现了MiddlewareInitializerInterface接口 // [!code focus:2]
class Server implements OnRequestInterface, MiddlewareInitializerInterface 
{
    protected array $middlewares = [];

    protected ?CoreMiddlewareInterface $coreMiddleware = null;

    protected array $exceptionHandlers = [];

    protected ?string $serverName = null;

    protected ?EventDispatcherInterface $event = null;

    protected ?Option $option = null;

    public function __construct(
        protected ContainerInterface $container,
        protected HttpDispatcher $dispatcher,
        protected ExceptionHandlerDispatcher $exceptionHandlerDispatcher,
        protected ResponseEmitter $responseEmitter
    ) {
        if ($this->container->has(EventDispatcherInterface::class)) {
            $this->event = $this->container->get(EventDispatcherInterface::class);
        }
    }
    //....
}

```

:::
所以该类会调用`initCoreMiddleware`方法，进行中间件的初始化。

```php
$class->initCoreMiddleware($serverName);
```

> 文件位置：/vendor/hyperf/http-server/src/Server.php

```php
public function initCoreMiddleware(string $serverName): void
{
  // http
  $this->serverName = $serverName;
  // 创建核心中间件对象实例
  $this->coreMiddleware = $this->createCoreMiddleware();

  $config = $this->container->get(ConfigInterface::class);
  // 获取配置的中间件
  $this->middlewares = $config->get('middlewares.' . $serverName, []);
  // 设置异常处理器
  $this->exceptionHandlers = $config->get('exceptions.handler.' . $serverName, $this->getDefaultExceptionHandler());
  // 设置服务的option选项，对中间件进行排序
  $this->initOption();
}
```

## 创建中间件对象实例

首先创建核心中间件对象实例，通过`make`方法创建。

```php
protected function createCoreMiddleware(): CoreMiddlewareInterface // [!code focus:4]
{
    return make(CoreMiddleware::class, [$this->container, $this->serverName]);
}


function make(string $name, array $parameters = [])
{
    if (ApplicationContext::hasContainer()) {
        /** @var \Hyperf\Di\Container $container */
        $container = ApplicationContext::getContainer();
        if (method_exists($container, 'make')) {
            return $container->make($name, $parameters);
        }
    }
    $parameters = array_values($parameters);
    return new $name(...$parameters);
}

protected function initOption(): void
{
  $ports = $this->container->get(ServerFactory::class)->getConfig()?->getServers();
  if (! $ports) {
      return;
  }

  foreach ($ports as $port) {
      if ($port->getName() === $this->serverName) {
          $this->option = $port->getOptions();
      }
  }

  $this->option ??= Option::make([]);
  $this->option->setMustSortMiddlewaresByMiddlewares($this->middlewares);
}
```

实际上`make`方法还是从依赖注入容器中使用`make`方法创建对象。

### CoreMiddleware实例化过程

::: code-group
```php [构造函数]
public function __construct(protected ContainerInterface $container, private string $serverName)
{
     // 创建调度器
    $this->dispatcher = $this->createDispatcher($serverName);
    // 获取标准化接口实例
    $this->normalizer = $this->container->get(NormalizerInterface::class);
    // 获取方法定义收集器实例
    $this->methodDefinitionCollector = $this->container->get(MethodDefinitionCollectorInterface::class);
    // 如果存在，获取闭包定义收集器实例
    if ($this->container->has(ClosureDefinitionCollectorInterface::class)) {
        $this->closureDefinitionCollector = $this->container->get(ClosureDefinitionCollectorInterface::class);
    }
}


protected function createDispatcher(string $serverName): Dispatcher
{
    // 从容器中获取DispatcherFactory实例
    $factory = $this->container->get(DispatcherFactory::class);
    // 获取调度器实例
    return $factory->getDispatcher($serverName);
}
```
```php [CoreMiddleware类源码]
namespace Hyperf\HttpServer;

use Closure;
use FastRoute\Dispatcher;
use Hyperf\Codec\Json;
use Hyperf\Context\Context;
use Hyperf\Context\RequestContext;
use Hyperf\Context\ResponseContext;
use Hyperf\Contract\Arrayable;
use Hyperf\Contract\Jsonable;
use Hyperf\Contract\NormalizerInterface;
use Hyperf\Di\ClosureDefinitionCollectorInterface;
use Hyperf\Di\MethodDefinitionCollectorInterface;
use Hyperf\Di\ReflectionType;
use Hyperf\HttpMessage\Exception\MethodNotAllowedHttpException;
use Hyperf\HttpMessage\Exception\NotFoundHttpException;
use Hyperf\HttpMessage\Exception\ServerErrorHttpException;
use Hyperf\HttpMessage\Server\ResponsePlusProxy;
use Hyperf\HttpMessage\Stream\SwooleStream;
use Hyperf\HttpServer\Contract\CoreMiddlewareInterface;
use Hyperf\HttpServer\Router\Dispatched;
use Hyperf\HttpServer\Router\DispatcherFactory;
use Hyperf\Server\Exception\ServerException;
use InvalidArgumentException;
use Psr\Container\ContainerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\RequestHandlerInterface;
use RuntimeException;
use Swow\Psr7\Message\ResponsePlusInterface;

class CoreMiddleware implements CoreMiddlewareInterface
{
    protected Dispatcher $dispatcher;

    private MethodDefinitionCollectorInterface $methodDefinitionCollector;

    private ?ClosureDefinitionCollectorInterface $closureDefinitionCollector = null;

    private NormalizerInterface $normalizer;

    public function __construct(protected ContainerInterface $container, private string $serverName)
    {
        $this->dispatcher = $this->createDispatcher($serverName);
        $this->normalizer = $this->container->get(NormalizerInterface::class);
        $this->methodDefinitionCollector = $this->container->get(MethodDefinitionCollectorInterface::class);
        if ($this->container->has(ClosureDefinitionCollectorInterface::class)) {
            $this->closureDefinitionCollector = $this->container->get(ClosureDefinitionCollectorInterface::class);
        }
    }

    public function dispatch(ServerRequestInterface $request): ServerRequestInterface
    {
        $routes = $this->dispatcher->dispatch($request->getMethod(), $request->getUri()->getPath());

        $dispatched = new Dispatched($routes, $this->serverName);

        return RequestContext::set($request)->setAttribute(Dispatched::class, $dispatched);
    }

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $request = RequestContext::set($request);

        /** @var Dispatched $dispatched */
        $dispatched = $request->getAttribute(Dispatched::class);

        if (! $dispatched instanceof Dispatched) {
            throw new ServerException(sprintf('The dispatched object is not a %s object.', Dispatched::class));
        }

        $response = match ($dispatched->status) {
            Dispatcher::NOT_FOUND => $this->handleNotFound($request),
            Dispatcher::METHOD_NOT_ALLOWED => $this->handleMethodNotAllowed($dispatched->params, $request),
            Dispatcher::FOUND => $this->handleFound($dispatched, $request),
            default => null,
        };

        if (! $response instanceof ResponsePlusInterface) {
            $response = $this->transferToResponse($response, $request);
        }

        return $response->addHeader('Server', 'Hyperf');
    }

    public function getMethodDefinitionCollector(): MethodDefinitionCollectorInterface
    {
        return $this->methodDefinitionCollector;
    }

    public function getClosureDefinitionCollector(): ClosureDefinitionCollectorInterface
    {
        return $this->closureDefinitionCollector;
    }

    public function getNormalizer(): NormalizerInterface
    {
        return $this->normalizer;
    }

    protected function createDispatcher(string $serverName): Dispatcher
    {
        $factory = $this->container->get(DispatcherFactory::class);
        return $factory->getDispatcher($serverName);
    }

    /**
     * Handle the response when found.
     *
     * @return array|Arrayable|mixed|ResponseInterface|string
     */
    protected function handleFound(Dispatched $dispatched, ServerRequestInterface $request): mixed
    {
        if ($dispatched->handler->callback instanceof Closure) {
            $parameters = $this->parseClosureParameters($dispatched->handler->callback, $dispatched->params);
            $callback = $dispatched->handler->callback;
            $response = $callback(...$parameters);
        } else {
            [$controller, $action] = $this->prepareHandler($dispatched->handler->callback);
            $controllerInstance = $this->container->get($controller);
            if (! method_exists($controllerInstance, $action)) {
                // Route found, but the handler does not exist.
                throw new ServerErrorHttpException('Method of class does not exist.');
            }
            $parameters = $this->parseMethodParameters($controller, $action, $dispatched->params);
            $response = $controllerInstance->{$action}(...$parameters);
        }
        return $response;
    }

    /**
     * Handle the response when cannot found any routes.
     */
    protected function handleNotFound(ServerRequestInterface $request): mixed
    {
        throw new NotFoundHttpException();
    }

    /**
     * Handle the response when the routes found but doesn't match any available methods.
     */
    protected function handleMethodNotAllowed(array $methods, ServerRequestInterface $request): mixed
    {
        throw new MethodNotAllowedHttpException('Allow: ' . implode(', ', $methods));
    }

    protected function prepareHandler(array|string $handler): array
    {
        if (is_string($handler)) {
            if (str_contains($handler, '@')) {
                return explode('@', $handler);
            }
            if (str_contains($handler, '::')) {
                return explode('::', $handler);
            }
            return [$handler, '__invoke'];
        }
        if (is_array($handler) && isset($handler[0], $handler[1])) {
            return $handler;
        }
        throw new RuntimeException('Handler not exist.');
    }

    /**
     * Transfer the non-standard response content to a standard response object.
     *
     * @param null|array|Arrayable|Jsonable|ResponseInterface|string $response
     */
    protected function transferToResponse($response, ServerRequestInterface $request): ResponsePlusInterface
    {
        if (is_string($response)) {
            return $this->response()->addHeader('content-type', 'text/plain')->setBody(new SwooleStream($response));
        }

        if ($response instanceof ResponseInterface) {
            return new ResponsePlusProxy($response);
        }

        if (is_array($response) || $response instanceof Arrayable) {
            return $this->response()
                ->addHeader('content-type', 'application/json')
                ->setBody(new SwooleStream(Json::encode($response)));
        }

        if ($response instanceof Jsonable) {
            return $this->response()
                ->addHeader('content-type', 'application/json')
                ->setBody(new SwooleStream((string) $response));
        }

        if ($this->response()->hasHeader('content-type')) {
            return $this->response()->setBody(new SwooleStream((string) $response));
        }

        return $this->response()->addHeader('content-type', 'text/plain')->setBody(new SwooleStream((string) $response));
    }

    /**
     * Get response instance from context.
     */
    protected function response(): ResponsePlusInterface
    {
        return ResponseContext::get();
    }

    /**
     * Parse the parameters of method definitions, and then bind the specified arguments or
     * get the value from DI container, combine to an argument array that should be injected
     * and return the array.
     */
    protected function parseMethodParameters(string $controller, string $action, array $arguments): array
    {
        $definitions = $this->getMethodDefinitionCollector()->getParameters($controller, $action);
        return $this->getInjections($definitions, "{$controller}::{$action}", $arguments);
    }

    /**
     * Parse the parameters of closure definitions, and then bind the specified arguments or
     * get the value from DI container, combine to an argument array that should be injected
     * and return the array.
     */
    protected function parseClosureParameters(Closure $closure, array $arguments): array
    {
        if (! $this->container->has(ClosureDefinitionCollectorInterface::class)) {
            return [];
        }
        $definitions = $this->getClosureDefinitionCollector()->getParameters($closure);
        return $this->getInjections($definitions, 'Closure', $arguments);
    }

    /**
     * @param ReflectionType[] $definitions
     */
    private function getInjections(array $definitions, string $callableName, array $arguments): array
    {
        $injections = [];
        foreach ($definitions as $pos => $definition) {
            $value = $arguments[$pos] ?? $arguments[$definition->getMeta('name')] ?? null;
            if ($value === null) {
                if ($definition->getMeta('defaultValueAvailable')) {
                    $injections[] = $definition->getMeta('defaultValue');
                } elseif ($this->container->has($definition->getName())) {
                    $injections[] = $this->container->get($definition->getName());
                } elseif ($definition->allowsNull()) {
                    $injections[] = null;
                } else {
                    throw new InvalidArgumentException("Parameter '{$definition->getMeta('name')}' "
                        . "of {$callableName} should not be null");
                }
            } else {
                $injections[] = $this->getNormalizer()->denormalize($value, $definition->getName());
            }
        }
        return $injections;
    }
}

```
:::

`DispatcherFactory`类在实例化时，调用构造方法，进行初始化路由和调度器
```php
public function __construct()
{
    // 根据注解初始化路由
    $this->initAnnotationRoute(AnnotationCollector::list());
    // 根据配置文件初始化路由
    $this->initConfigRoute();
}

public function getDispatcher(string $serverName): Dispatcher
{
    if (isset($this->dispatchers[$serverName])) {
        return $this->dispatchers[$serverName];
    }

    $router = $this->getRouter($serverName);
    return $this->dispatchers[$serverName] = new GroupCountBased($router->getData());
}
```
CoreMiddleware实例化完成。

```php

public function initCoreMiddleware(string $serverName): void
{
    $this->serverName = $serverName;
    // 这里保存了所有的路由信息
    $this->coreMiddleware = $this->createCoreMiddleware();
    // 获取配置文件
    $config = $this->container->get(ConfigInterface::class); // [!code focus:5]
    // 设置配置的中间件数组
    $this->middlewares = $config->get('middlewares.' . $serverName, []);
    // 设置异常处理器
    $this->exceptionHandlers = $config->get('exceptions.handler.' . $serverName, $this->getDefaultExceptionHandler());

    $this->initOption();
}
```
### 设置异常处理器
配置服务的异常处理程序，首先从配置文件`exceptions`中，获取配置的异常处理器，如不存在，则使用默认的异常处理器。
```php
$this->exceptionHandlers = $config->get('exceptions.handler.' . $serverName, $this->getDefaultExceptionHandler());

protected function getDefaultExceptionHandler(): array
{
    return [
        HttpExceptionHandler::class,
    ];
}
```

### 初始化配置
设置`Server`的`option`参数
```php

protected function initOption(): void
{
    // 获取服务器配置对象数组，类型为port[]
    $ports = $this->container->get(ServerFactory::class)->getConfig()?->getServers();
    if (! $ports) {
        return;
    }
    // 设置服务的option配置
    foreach ($ports as $port) {
        if ($port->getName() === $this->serverName) {
            $this->option = $port->getOptions();
        }
    }

    $this->option ??= Option::make([]);
    // 对中间件进行排序
    $this->option->setMustSortMiddlewaresByMiddlewares($this->middlewares);
}

// 如果设置了顺序比如[foo::class => 1]或者继承了PriorityMiddleware中间件，都将启动中间件的排序操作
public function setMustSortMiddlewaresByMiddlewares(array $middlewares): static
{
    foreach ($middlewares as $middleware) {
        if (is_int($middleware) || $middleware instanceof PriorityMiddleware) {
            return $this->setMustSortMiddlewares(true);
        }
    }
    return $this;
}
```
最后，初始化中间件完成，等待`http`请求。